@@ -202,10 +202,12 @@ bool read_multi_label_template
     size_large &nrows,
     size_large &ncols,
     size_large &nclasses,
+    const size_t limit_nrows,
     const bool ignore_zero_valued,
     const bool sort_indices,
     const bool text_is_base1,
-    const bool assume_no_qid
+    const bool assume_no_qid,
+    const bool assume_trailing_ws
 )
 {
     if (input_file.fail())
@@ -351,6 +353,8 @@ bool read_multi_label_template
             #if defined(HAS_Z_SPECIFIER) && defined(PLATFORM_IS_64_OR_HIGHER)
             std::sscanf(ln.c_str(), "%zu %zu %zu",
                         &header_nrows, &header_ncols, &header_nclasses);
+            if (limit_nrows)
+                header_nrows = std::min(header_nrows, limit_nrows);
             bool size_is_within_type
                 =
             check_size_is_within_type<int_t>(header_nrows, header_ncols, header_nclasses);
@@ -358,6 +362,8 @@ bool read_multi_label_template
             uint64_t temp1, temp2, temp3;
             std::sscanf(ln.c_str(), "%" SCNu64 " %" SCNu64 " %" SCNu64,
                         &temp1, &temp2, &temp3);
+            if (limit_nrows)
+                temp1 = std::min(temp1, (uint64_t)limit_nrows);
             bool size_is_within_type
                 =
             check_size_is_within_type<int_t>(temp1, temp2, temp3);
@@ -472,6 +478,8 @@ bool read_multi_label_template
 
         next_line:
         indptr.push_back(indices.size());
+        if (limit_nrows && indptr.size()-(size_t)1 == limit_nrows)
+            break;
     }
 
     if (text_is_base1) {
@@ -510,10 +518,12 @@ bool read_multi_label_template
     size_large &nrows,
     size_large &ncols,
     size_large &nclasses,
+    const size_t limit_nrows,
     const bool ignore_zero_valued,
     const bool sort_indices,
     const bool text_is_base1,
-    const bool assume_no_qid
+    const bool assume_no_qid,
+    const bool assume_trailing_ws
 )
 {
     if (input_file == NULL)
@@ -679,6 +689,16 @@ bool read_multi_label_template
                 continue;
             }
 
+            else if (limit_nrows)
+            {
+                #if defined(HAS_Z_SPECIFIER) && defined(PLATFORM_IS_64_OR_HIGHER)
+                header_nrows = std::min(header_nrows, limit_nrows);
+                #else
+                temp1 = std::min(temp1, (uint64_t)limit_nrows);
+                header_nrows = temp1;
+                #endif
+            }
+
             bool size_is_within_type
                 =
             #if defined(HAS_Z_SPECIFIER) && defined(PLATFORM_IS_64_OR_HIGHER)
@@ -807,16 +827,45 @@ bool read_multi_label_template
                 qid.push_back(curr_col);
         }
 
-        while ((n_matched = fscanf(input_file, parse_code_features, &curr_col, &curr_val)) == 2)
+        n_matched = 0;
+        if (assume_trailing_ws)
         {
-            if (!ignore_zero_valued || curr_val)
+            while ((n_matched = fscanf(input_file, parse_code_features, &curr_col, &curr_val)) == 2)
             {
-                indices.push_back(curr_col);
-                values.push_back(curr_val);
+                if (!ignore_zero_valued || curr_val)
+                {
+                    indices.push_back(curr_col);
+                    values.push_back(curr_val);
+                }
+                
+                return_to = ftell_(input_file);
+                do { next_char = fgetc(input_file); }
+                while (next_char != EOF && isspace(next_char) && (char)next_char != '\n');
+                if (next_char == EOF || (char)next_char == '\n' || (char)next_char == '#')
+                    goto next_line;
+                else
+                    fseek_(input_file, return_to, SEEK_SET);
+            }
+            if (n_matched == 1) {
+                fseek_(input_file, return_to, SEEK_SET);
+                goto next_line;
+            }
+        }
+
+        else
+        {
+            while ((n_matched = fscanf(input_file, parse_code_features, &curr_col, &curr_val)) == 2)
+            {
+                if (!ignore_zero_valued || curr_val)
+                {
+                    indices.push_back(curr_col);
+                    values.push_back(curr_val);
+                }
+                
+                next_char = fgetc(input_file);
+                if ((char)next_char == '\n' || (char)next_char == '\r' || !isspace((char)next_char))
+                    break;
             }
-            next_char = fgetc(input_file);
-            if ((char)next_char == '\n' || (char)next_char == '\r' || !isspace((char)next_char))
-                break;
         }
 
 
@@ -831,16 +880,20 @@ bool read_multi_label_template
 
         if (n_matched == EOF || next_char == EOF)
             break;
-        else if ((char)next_char == '\n')
-            continue;
-        else {
-            do { next_char = fgetc(input_file); }
-            while ((char)next_char != '\n' && next_char != EOF);
-            if (next_char == EOF)
-                break;
+        while (next_char != EOF && (char)next_char != '\n')
+        {
+            next_char = fgetc(input_file);
         }
+
+        if (limit_nrows && indptr.size()-1 == limit_nrows)
+            break;
     }
 
+    if (!feof(input_file) && ferror(input_file))
+    {
+        print_errno();
+        return false;
+    }
 
     if (text_is_base1) {
         subtract_one_from_vec(indices);
@@ -876,10 +929,12 @@ bool read_single_label_template
     size_large &nrows,
     size_large &ncols,
     size_large &nclasses,
+    const size_t limit_nrows,
     const bool ignore_zero_valued,
     const bool sort_indices,
     const bool text_is_base1,
-    const bool assume_no_qid
+    const bool assume_no_qid,
+    const bool assume_trailing_ws
 )
 {
     if (input_file.fail())
@@ -1092,6 +1147,8 @@ bool read_single_label_template
             #if defined(HAS_Z_SPECIFIER) && defined(PLATFORM_IS_64_OR_HIGHER)
             std::sscanf(ln.c_str(), "%zu %zu %zu",
                         &header_nrows, &header_ncols, &header_nclasses);
+            if (limit_nrows)
+                header_nrows = std::min(header_nrows, limit_nrows);
             bool size_is_within_type
                 =
             check_size_is_within_type<int_t>(header_nrows, header_ncols, header_nclasses);
@@ -1099,6 +1156,8 @@ bool read_single_label_template
             uint64_t temp1, temp2, temp3;
             std::sscanf(ln.c_str(), "%" SCNu64 " %" SCNu64 " %" SCNu64,
                         &temp1, &temp2, &temp3);
+            if (limit_nrows)
+                temp1 = std::min(temp1, (uint64_t)limit_nrows);
             bool size_is_within_type
                 =
             check_size_is_within_type<int_t>(temp1, temp2, temp3);
@@ -1208,6 +1267,8 @@ bool read_single_label_template
 
         next_line:
         indptr.push_back(indices.size());
+        if (limit_nrows && indptr.size()-1 == limit_nrows)
+            break;
     }
 
     if (text_is_base1)
@@ -1243,10 +1304,12 @@ bool read_single_label_template
     size_large &nrows,
     size_large &ncols,
     size_large &nclasses,
+    const size_t limit_nrows,
     const bool ignore_zero_valued,
     const bool sort_indices,
     const bool text_is_base1,
-    const bool assume_no_qid
+    const bool assume_no_qid,
+    const bool assume_trailing_ws
 )
 {
     if (input_file == NULL)
@@ -1480,6 +1543,16 @@ bool read_single_label_template
                 continue;
             }
 
+            else if (limit_nrows)
+            {
+                #if defined(HAS_Z_SPECIFIER) && defined(PLATFORM_IS_64_OR_HIGHER)
+                header_nrows = std::min(header_nrows, limit_nrows);
+                #else
+                temp1 = std::min(temp1, (uint64_t)limit_nrows);
+                header_nrows = temp1;
+                #endif
+            }
+
             bool size_is_within_type
                 =
             #if defined(HAS_Z_SPECIFIER) && defined(PLATFORM_IS_64_OR_HIGHER)
@@ -1611,18 +1684,45 @@ bool read_single_label_template
                 qid.push_back(curr_col);
         }
 
-        while ((n_matched = fscanf(input_file, parse_code_features, &curr_col, &curr_val)) == 2)
+        if (assume_trailing_ws)
         {
-            if (!ignore_zero_valued || curr_val)
+            while ((n_matched = fscanf(input_file, parse_code_features, &curr_col, &curr_val)) == 2)
             {
-                indices.push_back(curr_col);
-                values.push_back(curr_val);
+                if (!ignore_zero_valued || curr_val)
+                {
+                    indices.push_back(curr_col);
+                    values.push_back(curr_val);
+                }
+                
+                return_to = ftell_(input_file);
+                do { next_char = fgetc(input_file); }
+                while (next_char != EOF && isspace(next_char) && (char)next_char != '\n');
+                if (next_char == EOF || (char)next_char == '\n' || (char)next_char == '#')
+                    goto next_line;
+                else
+                    fseek_(input_file, return_to, SEEK_SET);
+            }
+                if (n_matched == 1) {
+                fseek_(input_file, return_to, SEEK_SET);
+                goto next_line;
             }
-            next_char = fgetc(input_file);
-            if ((char)next_char == '\n' || (char)next_char == '\r' || !isspace((char)next_char))
-                break;
         }
 
+        else
+        {
+             while ((n_matched = fscanf(input_file, parse_code_features, &curr_col, &curr_val)) == 2)
+            {
+                if (!ignore_zero_valued || curr_val)
+                {
+                    indices.push_back(curr_col);
+                    values.push_back(curr_val);
+                }
+                
+                next_char = fgetc(input_file);
+                if ((char)next_char == '\n' || (char)next_char == '\r' || !isspace((char)next_char))
+                    break;
+                }
+        }
 
         next_line:
         indptr.push_back(indices.size());
@@ -1635,14 +1735,19 @@ bool read_single_label_template
 
         if (n_matched == EOF || next_char == EOF)
             break;
-        else if ((char)next_char == '\n')
-            continue;
-        else {
-            do { next_char = fgetc(input_file); }
-            while ((char)next_char != '\n' && next_char != EOF);
-            if (next_char == EOF)
-                break;
+        while (next_char != EOF && (char)next_char != '\n')
+        {
+            next_char = fgetc(input_file);
         }
+
+        if (limit_nrows && indptr.size()-1 == limit_nrows)
+            break;
+    }
+
+    if (!feof(input_file) && ferror(input_file))
+    {
+        print_errno();
+        return false;
     }
 
     if (text_is_base1)
