@@ -28,11 +28,18 @@
 #include "readsparse_internal.hpp"
 
 #ifndef _FOR_R
-#   define throw_err(msg) {fprintf(stderr, msg); fflush(stderr);}
+#   define throw_err(msg) {fprintf(stderr, "%s", msg); fflush(stderr);}
 #else
-#   define throw_err(msg) REprintf(msg)
+#   define throw_err(msg) {REprintf(msg); R_FlushConsole();}
 #endif
 
+static void print_errno()
+{
+    char msg[1000];
+    snprintf(msg, 999, "Error %d: %s\n", errno, strerror(errno));
+    throw_err(msg);
+}
+
 
 template <class int_t>
 bool check_is_sorted(int_t* vec, size_t n)
