@@ -518,8 +518,7 @@ bool write_multi_label_template
 
     throw_err:
     {
-        char msg[1000];
-        snprintf(msg, 999, "Error %d: %s\n", errno, strerror(errno));
+        print_errno();
         return false;
     }
 }
@@ -928,7 +927,7 @@ bool write_single_label_template
             #endif
             {
                 #ifdef HAS_FULL_STDIO
-                #   if defined(__GNUC__) || defined(__GNUG__)
+                #   if (defined(__GNUC__) || defined(__GNUG__)) && (__GNUC__ >= 5)
                 #       pragma GCC diagnostic push
                 #       pragma GCC diagnostic ignored "-Wformat"
                 #   elif defined(__clang__)
@@ -936,7 +935,7 @@ bool write_single_label_template
                 #       pragma clang diagnostic ignored "-Wformat"
                 #   endif
                 succeded = fprintf(output_file, label_specifier, decimal_places, labels[row]);
-                #   if defined(__GNUC__) || defined(__GNUG__)
+                #   if (defined(__GNUC__) || defined(__GNUG__)) && (__GNUC__ >= 5)
                 #       pragma GCC diagnostic pop
                 #   elif defined(__clang__)
                 #       pragma clang diagnostic pop
@@ -1069,8 +1068,7 @@ bool write_single_label_template
 
     throw_err:
     {
-        char msg[1000];
-        snprintf(msg, 999, "Error %d: %s\n", errno, strerror(errno));
+        print_errno();
         return false;
     }
 }
